package org.openxma.dsl.core.util;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.Enumerator;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EReference;
import org.openxma.dsl.core.model.AdditiveExpr;
import org.openxma.dsl.core.model.AdditiveRights;
import org.openxma.dsl.core.model.AtomicBoolExpr;
import org.openxma.dsl.core.model.AtomicExpr;
import org.openxma.dsl.core.model.CondANDExpr;
import org.openxma.dsl.core.model.CondANDRights;
import org.openxma.dsl.core.model.CondORExpr;
import org.openxma.dsl.core.model.CondORRights;
import org.openxma.dsl.core.model.EqualityExpr;
import org.openxma.dsl.core.model.EqualityOp;
import org.openxma.dsl.core.model.MultiplicativeExpr;
import org.openxma.dsl.core.model.MultiplicativeRights;
import org.openxma.dsl.core.model.RelationalExpr;
import org.openxma.dsl.core.model.RelationalOp;

public class NextOperatorSwitch extends CoreSwitch<String> {
	EObject current;

	@Override
	public String caseAtomicExpr(AtomicExpr object) {
		current = object;
		return doSwitch(object.eContainer());
	}
	

	@Override
	public String caseMultiplicativeRights(MultiplicativeRights object) {
		current = object;
		return doSwitch(object.eContainer());
	}

	@Override
	public String caseMultiplicativeExpr(MultiplicativeExpr object) {
		if (current instanceof AtomicExpr) {
			if (object.getRights().isEmpty()) {
				current = object;
				return doSwitch(object.eContainer());
			} else {
				return object.getRights().iterator().next().getOp().getLiteral();
			}
		} else {
			return getNextOperatorOrSwitchContainer(object);
		}
	}

	@Override
	public String caseAdditiveRights(AdditiveRights object) {
		current = object;
		return doSwitch(object.eContainer());
	}

	@Override
	public String caseAdditiveExpr(AdditiveExpr object) {
		if (current instanceof MultiplicativeExpr) {
			if (object.getRights().isEmpty()) {
				current = object;
				return doSwitch(object.eContainer());
			} else {
				return object.getRights().iterator().next().getOp().getLiteral();
			}
		} else {
			return getNextOperatorOrSwitchContainer(object);
		}
	}

	@Override
	public String caseRelationalExpr(RelationalExpr object) {
		if (object.getRight()!=null && object.getRight().equals(current)) {
			current = object;
			return doSwitch(object.eContainer());
		} else {
			if (!object.getOp().equals(RelationalOp.NULL)) {
				return object.getOp().getLiteral();
			} else {
				current = object;
				return doSwitch(object.eContainer());
			}
		}
	}

	@Override
	public String caseCondANDRights(CondANDRights object) {
		current = object;
		return doSwitch(object.eContainer());
	}

	@Override
	public String caseCondANDExpr(CondANDExpr object) {
		if (current instanceof AtomicBoolExpr) {
			if (object.getRights().isEmpty()) {
				current = object;
				return doSwitch(object.eContainer());
			} else {
				return object.getRights().iterator().next().getOp().getLiteral();
			}
		} else {

			return getNextOperatorOrSwitchContainer(object);
		}
	}

	@Override
	public String caseCondORRights(CondORRights object) {
		current = object;
		return doSwitch(object.eContainer());
	}

	@Override
	public String caseCondORExpr(CondORExpr object) {
		if (current instanceof CondANDExpr) {
			if (object.getRights().isEmpty()) {
				current = object;
				return doSwitch(object.eContainer());
			} else {
				return object.getRights().iterator().next().getOp().getLiteral();
			}
		} else {
			return getNextOperatorOrSwitchContainer(object);
		}
	}

	@Override
	public String caseEqualityExpr(EqualityExpr object) {
		if (current.equals(object.getRight())) {
			return "";
		} else {
			if (!object.getOp().equals(EqualityOp.NULL)) {
				return object.getOp().getLiteral();
			} else {
				return "";
			}
		}
	}

	private String getNextOperatorOrSwitchContainer(EObject eObject) {
		EReference rights = (EReference) eObject.eClass()
				.getEStructuralFeature("rights");
		EList list = (EList) eObject.eGet(rights);
		int indexOf = list.indexOf(current);
		if (indexOf < (list.size() - 1)) {
			EObject right = (EObject) list.get(indexOf + 1);
			EAttribute operator = (EAttribute) right.eClass()
					.getEStructuralFeature("op");
			Enumerator enumerator = (Enumerator) right.eGet(operator);
			return enumerator.getLiteral();
		} else {
			current = eObject;
			return doSwitch(eObject.eContainer());
		}
	}

}
